#ifndef dynpool_hh
#define dynpool_hh

#include "dynlib.hh"
#include "autoptr"

class dynlib : public ptrable
{
private:
	void *handle;
	const char *libname;
public:
	bool ok() const { return handle; }
	explicit dynlib(const char *fn);
	~dynlib();
	void *getsym(const char *sym);
};

class dynclass : public dyncontent, public ptrable
{
public:
	explicit dynclass(autoptr<dynlib> &l);
	~dynclass();
private:
	friend class dynpool;
	autoptr<dynlib> lib;
	void operator=(const dyncontent &p);
};

class dynpool
{
public:
	typedef class dynclass ent;

	dynpool();
	~dynpool();
	
	void load(const char *fn);
	
	/* Number of classes loaded */
	unsigned size() const;
	
	const ent *find(const char *name) const;
	const ent &operator[] (unsigned ind) const;
	
	void unregister(const char *name);

private:
	class dyn_NameList *hash;
	
	void Exception(const dynclass &old, const dyncontent &prob);
	void Exception(const char *dllname);
	void Exception(const char *dllname, const char *symbol);
};

#endif
